@@ -1,10 +1,7 @@
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
 const Clutter = imports.gi.Clutter;
-const Gtk = imports.gi.Gtk;
 const Mainloop = imports.mainloop;
-const Pango = imports.gi.Pango;
-const Shell = imports.gi.Shell;
 const Signals = imports.signals;
 const Lang = imports.lang;
 const St = imports.gi.St;
@@ -15,16 +12,14 @@ const AppDisplay = imports.ui.appDisplay;
 const DND = imports.ui.dnd;
 const DocDisplay = imports.ui.docDisplay;
 const PlaceDisplay = imports.ui.placeDisplay;
-const GenericDisplay = imports.ui.genericDisplay;
 const Main = imports.ui.main;
+const Overview = imports.ui.overview;
 const Search = imports.ui.search;
+const Tweener = imports.ui.tweener;
 
 // 25 search results (per result type) should be enough for everyone
 const MAX_RENDERED_SEARCH_RESULTS = 25;
 
-const DOCS = 'docs';
-const PLACES = 'places';
-
 /*
  * Returns the index in an array of a given length that is obtained
  * if the provided index is incremented by an increment and the array
@@ -38,14 +33,6 @@ function _getIndexWrapped(index, increment, length) {
    return (index + increment + length) % length;
 }
 
-function _createDisplay(displayType, flags) {
-    if (displayType == DOCS)
-        return new DocDisplay.DocDisplay(flags);
-    else if (displayType == PLACES)
-        return new PlaceDisplay.PlaceDisplay(flags);
-    return null;
-}
-
 function Pane() {
     this._init();
 }
@@ -62,15 +49,6 @@ Pane.prototype = {
             return true;
         }));
 
-        let chromeTop = new St.BoxLayout();
-
-        let dummy = new St.Bin();
-        chromeTop.add(dummy, { expand: true });
-        this.actor.add(chromeTop);
-
-        this.content = new St.BoxLayout({ vertical: true });
-        this.actor.add(this.content, { expand: true });
-
         // Hidden by default
         this.actor.hide();
     },
@@ -79,20 +57,33 @@ Pane.prototype = {
         if (this._open)
             return;
         this._open = true;
-        this.actor.show();
         this.emit('open-state-changed', this._open);
+        this.actor.opacity = 0;
+        this.actor.show();
+        Tweener.addTween(this.actor,
+                         { opacity: 255,
+                           time: Overview.PANE_FADE_TIME,
+                           transition: 'easeOutQuad'
+                         });
     },
 
     close: function () {
         if (!this._open)
             return;
         this._open = false;
-        this.actor.hide();
-        this.emit('open-state-changed', this._open);
+        Tweener.addTween(this.actor,
+                         { opacity: 0,
+                           time: Overview.PANE_FADE_TIME,
+                           transition: 'easeOutQuad',
+                           onComplete: Lang.bind(this, function() {
+                               this.actor.hide();
+                               this.emit('open-state-changed', this._open);
+                           })
+                         });
     },
 
     destroyContent: function() {
-        let children = this.content.get_children();
+        let children = this.actor.get_children();
         for (let i = 0; i < children.length; i++) {
             children[i].destroy();
         }
@@ -107,51 +98,22 @@ Pane.prototype = {
 };
 Signals.addSignalMethods(Pane.prototype);
 
-function ResultArea(displayType, flags) {
-    this._init(displayType, flags);
+function ResultArea() {
+    this._init();
 }
 
 ResultArea.prototype = {
-    _init : function(displayType, flags) {
+    _init : function() {
         this.actor = new St.BoxLayout({ vertical: true });
         this.resultsContainer = new St.BoxLayout({ style_class: 'dash-results-container' });
         this.actor.add(this.resultsContainer, { expand: true });
 
-        this.display = _createDisplay(displayType, flags);
+        this.display = new DocDisplay.DocDisplay();
         this.resultsContainer.add(this.display.actor, { expand: true });
         this.display.load();
     }
 };
 
-// Utility function shared between ResultPane and the DocDisplay in the main dash.
-// Connects to the detail signal of the display, and on-demand creates a new
-// pane.
-function createPaneForDetails(dash, display) {
-    let detailPane = null;
-    display.connect('show-details', Lang.bind(this, function(display, index) {
-        if (detailPane == null) {
-            detailPane = new Pane();
-            detailPane.connect('open-state-changed', Lang.bind(this, function (pane, isOpen) {
-                if (!isOpen) {
-                    /* Ensure we don't keep around large preview textures */
-                    detailPane.destroyContent();
-                }
-            }));
-            dash._addPane(detailPane, St.Align.START);
-        }
-
-        if (index >= 0) {
-            detailPane.destroyContent();
-            let details = display.createDetailsForIndex(index);
-            detailPane.content.add(details, { expand: true });
-            detailPane.open();
-        } else {
-            detailPane.close();
-        }
-    }));
-    return null;
-}
-
 function ResultPane(dash) {
     this._init(dash);
 }
@@ -161,21 +123,12 @@ ResultPane.prototype = {
 
     _init: function(dash) {
         Pane.prototype._init.call(this);
-        this._dash = dash;
-    },
 
-    // Create a display of displayType and pack it into this pane's
-    // content area.  Return the display.
-    packResults: function(displayType) {
-        let resultArea = new ResultArea(displayType);
-
-        createPaneForDetails(this._dash, resultArea.display);
-
-        this.content.add(resultArea.actor, { expand: true });
+        let resultArea = new ResultArea();
+        this.actor.add(resultArea.actor, { expand: true });
         this.connect('open-state-changed', Lang.bind(this, function(pane, isOpen) {
             resultArea.display.resetState();
         }));
-        return resultArea.display;
     }
 };
 
@@ -203,11 +156,21 @@ SearchEntry.prototype = {
             }));
         this.actor.connect('destroy', Lang.bind(this, this._onDestroy));
 
+        global.stage.connect('notify::key-focus', Lang.bind(this, this._updateCursorVisibility));
+
         this.pane = null;
 
         this._capturedEventId = 0;
     },
 
+    _updateCursorVisibility: function() {
+        let focus = global.stage.get_key_focus();
+        if (focus == global.stage || focus == this.entry)
+            this.entry.set_cursor_visible(true);
+        else
+            this.entry.set_cursor_visible(false);
+    },
+
     show: function() {
         if (this._capturedEventId == 0)
             this._capturedEventId = global.stage.connect('captured-event',
@@ -225,11 +188,15 @@ SearchEntry.prototype = {
         }
     },
 
-    setPane: function (pane) {
-        this._pane = pane;
-    },
-
     reset: function () {
+        let [x, y, mask] = global.get_pointer();
+        let actor = global.stage.get_actor_at_pos (Clutter.PickMode.REACTIVE,
+                                                   x, y);
+        // this.actor is never hovered directly, only its clutter_text and icon
+        let hovered = this.actor == actor.get_parent();
+
+        this.actor.set_hover(hovered);
+
         this.entry.text = '';
         global.stage.set_key_focus(null);
         this.entry.set_cursor_visible(true);
@@ -252,16 +219,7 @@ SearchEntry.prototype = {
 
     _onCapturedEvent: function(actor, event) {
         let source = event.get_source();
-        let panelEvent = false;
-
-        if (source) {
-            let parent = source;
-            do {
-                if (parent == Main.panel.actor)
-                    break;
-            } while ((parent = parent.get_parent()) != null);
-            panelEvent = (parent != null);
-        }
+        let panelEvent = source && Main.panel.actor.contains(source);
 
         switch (event.type()) {
             case Clutter.EventType.BUTTON_PRESS:
@@ -351,6 +309,14 @@ SearchResult.prototype = {
         this.actor.connect('clicked', Lang.bind(this, this._onResultClicked));
 
         let draggable = DND.makeDraggable(this.actor);
+        draggable.connect('drag-begin',
+                          Lang.bind(this, function() {
+                              Main.overview.beginItemDrag(this);
+                          }));
+        draggable.connect('drag-end',
+                          Lang.bind(this, function() {
+                              Main.overview.endItemDrag(this);
+                          }));
     },
 
     setSelected: function(selected) {
@@ -450,38 +416,40 @@ SearchResults.prototype = {
         this._selectedProvider = -1;
         this._providers = this._searchSystem.getProviders();
         this._providerMeta = [];
-        for (let i = 0; i < this._providers.length; i++) {
-            let provider = this._providers[i];
-            let providerBox = new St.BoxLayout({ style_class: 'dash-search-section',
-                                                  vertical: true });
-            let titleButton = new St.Button({ style_class: 'dash-search-section-header',
-                                              reactive: true,
-                                              x_fill: true,
-                                              y_fill: true });
-            titleButton.connect('clicked', Lang.bind(this, function () { this._onHeaderClicked(provider); }));
-            providerBox.add(titleButton);
-            let titleBox = new St.BoxLayout();
-            titleButton.set_child(titleBox);
-            let title = new St.Label({ text: provider.title });
-            let count = new St.Label();
-            titleBox.add(title, { expand: true });
-            titleBox.add(count);
+        for (let i = 0; i < this._providers.length; i++)
+            this.createProviderMeta(this._providers[i]);
+    },
 
-            let resultDisplayBin = new St.Bin({ style_class: 'dash-search-section-results',
-                                                x_fill: true,
-                                                y_fill: true });
-            providerBox.add(resultDisplayBin, { expand: true });
-            let resultDisplay = provider.createResultContainerActor();
-            if (resultDisplay == null) {
-                resultDisplay = new OverflowSearchResults(provider);
-            }
-            resultDisplayBin.set_child(resultDisplay.actor);
+    createProviderMeta: function(provider) {
+        let providerBox = new St.BoxLayout({ style_class: 'dash-search-section',
+                                             vertical: true });
+        let titleButton = new St.Button({ style_class: 'dash-search-section-header',
+                                          reactive: true,
+                                          x_fill: true,
+                                          y_fill: true });
+        titleButton.connect('clicked', Lang.bind(this, function () { this._onHeaderClicked(provider); }));
+        providerBox.add(titleButton);
+        let titleBox = new St.BoxLayout();
+        titleButton.set_child(titleBox);
+        let title = new St.Label({ text: provider.title });
+        let count = new St.Label();
+        titleBox.add(title, { expand: true });
+        titleBox.add(count);
 
-            this._providerMeta.push({ actor: providerBox,
-                                      resultDisplay: resultDisplay,
-                                      count: count });
-            this.actor.add(providerBox);
+        let resultDisplayBin = new St.Bin({ style_class: 'dash-search-section-results',
+                                            x_fill: true,
+                                            y_fill: true });
+        providerBox.add(resultDisplayBin, { expand: true });
+        let resultDisplay = provider.createResultContainerActor();
+        if (resultDisplay == null) {
+            resultDisplay = new OverflowSearchResults(provider);
         }
+        resultDisplayBin.set_child(resultDisplay.actor);
+
+        this._providerMeta.push({ actor: providerBox,
+                                  resultDisplay: resultDisplay,
+                                  count: count });
+        this.actor.add(providerBox);
     },
 
     _clearDisplay: function() {
@@ -617,6 +585,9 @@ MoreLink.prototype = {
     },
 
     activate: function() {
+        if (!this.actor.visible)
+            return true; // If the link isn't visible we don't want the header to react
+                         // to clicks
         if (this.pane == null) {
             // Ensure the pane is created; the activated handler will call setPane
             this.emit('activated');
@@ -638,18 +609,6 @@ MoreLink.prototype = {
 
 Signals.addSignalMethods(MoreLink.prototype);
 
-function BackLink() {
-    this._init();
-}
-
-BackLink.prototype = {
-    _init : function () {
-        this.actor = new St.Button({ style_class: 'section-header-back',
-                                      reactive: true });
-        this.actor.set_child(new St.Bin({ style_class: 'section-header-back-image' }));
-    }
-};
-
 function SectionHeader(title, suppressBrowse) {
     this._init(title, suppressBrowse);
 }
@@ -664,22 +623,11 @@ SectionHeader.prototype = {
         this._innerBox = new St.BoxLayout({ style_class: 'section-header-inner' });
         this.actor.set_child(this._innerBox);
 
-        this.backLink = new BackLink();
-        this._innerBox.add(this.backLink.actor);
-        this.backLink.actor.hide();
-        this.backLink.actor.connect('clicked', Lang.bind(this, function (actor) {
-            this.emit('back-link-activated');
-        }));
-
         let textBox = new St.BoxLayout({ style_class: 'section-text-content' });
         this.text = new St.Label({ style_class: 'section-title',
                                    text: title });
         textBox.add(this.text, { x_align: St.Align.START });
 
-        this.countText = new St.Label({ style_class: 'section-count' });
-        textBox.add(this.countText, { expand: true, x_fill: false, x_align: St.Align.END });
-        this.countText.hide();
-
         this._innerBox.add(textBox, { expand: true });
 
         if (!suppressBrowse) {
@@ -693,31 +641,11 @@ SectionHeader.prototype = {
         this.moreLink.activate();
     },
 
-    setTitle : function(title) {
-        this.text.text = title;
-    },
-
-    setBackLinkVisible : function(visible) {
-        if (visible)
-            this.backLink.actor.show();
-        else
-            this.backLink.actor.hide();
-    },
-
     setMoreLinkVisible : function(visible) {
         if (visible)
             this.moreLink.actor.show();
         else
             this.moreLink.actor.hide();
-    },
-
-    setCountText : function(countText) {
-        if (countText == '') {
-            this.countText.hide();
-        } else {
-            this.countText.show();
-            this.countText.text = countText;
-        }
     }
 };
 
@@ -882,7 +810,6 @@ Dash.prototype = {
         this._docsSection.header.moreLink.connect('activated', Lang.bind(this, function (link) {
             if (this._moreDocsPane == null) {
                 this._moreDocsPane = new ResultPane(this);
-                this._moreDocsPane.packResults(DOCS);
                 this._addPane(this._moreDocsPane, St.Align.END);
                 link.setPane(this._moreDocsPane);
            }
@@ -939,6 +866,13 @@ Dash.prototype = {
         return false;
     },
 
+    addSearchProvider: function(provider) {
+        //Add a new search provider to the dash.
+
+        this._searchSystem.registerProvider(provider);
+        this.searchResults.createProviderMeta(provider);
+    },
+
     show: function() {
         this._searchEntry.show();
         if (this._keyPressId == 0)
