@@ -1,7 +1,8 @@
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
-const Big = imports.gi.Big;
 const Clutter = imports.gi.Clutter;
+const GConf = imports.gi.GConf;
+const GLib = imports.gi.GLib;
 const Gio = imports.gi.Gio;
 const Pango = imports.gi.Pango;
 const St = imports.gi.St;
@@ -18,22 +19,22 @@ const Tweener = imports.ui.tweener;
 const Main = imports.ui.main;
 
 /* Imports...feel free to add here as needed */
-var commandHeader = "const Clutter = imports.gi.Clutter; " +
-                    "const GLib = imports.gi.GLib; " +
-                    "const Gtk = imports.gi.Gtk; " +
-                    "const Mainloop = imports.mainloop; " +
-                    "const Meta = imports.gi.Meta; " +
-                    "const Shell = imports.gi.Shell; " +
-                    "const Main = imports.ui.main; " +
-                    "const Lang = imports.lang; " +
-                    "const Tweener = imports.ui.tweener; " +
+var commandHeader = 'const Clutter = imports.gi.Clutter; ' +
+                    'const GLib = imports.gi.GLib; ' +
+                    'const Gtk = imports.gi.Gtk; ' +
+                    'const Mainloop = imports.mainloop; ' +
+                    'const Meta = imports.gi.Meta; ' +
+                    'const Shell = imports.gi.Shell; ' +
+                    'const Main = imports.ui.main; ' +
+                    'const Lang = imports.lang; ' +
+                    'const Tweener = imports.ui.tweener; ' +
                     /* Utility functions...we should probably be able to use these
                      * in the shell core code too. */
-                    "const stage = global.stage; " +
-                    "const color = function(pixel) { let c= new Clutter.Color(); c.from_pixel(pixel); return c; }; " +
+                    'const stage = global.stage; ' +
+                    'const color = function(pixel) { let c= new Clutter.Color(); c.from_pixel(pixel); return c; }; ' +
                     /* Special lookingGlass functions */
-                       "const it = Main.lookingGlass.getIt(); " +
-                    "const r = Lang.bind(Main.lookingGlass, Main.lookingGlass.getResult); ";
+                       'const it = Main.lookingGlass.getIt(); ' +
+                    'const r = Lang.bind(Main.lookingGlass, Main.lookingGlass.getResult); ';
 
 function Notebook() {
     this._init();
@@ -43,14 +44,16 @@ Notebook.prototype = {
     _init: function() {
         this.actor = new St.BoxLayout({ vertical: true });
 
-        this.tabControls = new St.BoxLayout({ style_class: "labels" });
+        this.tabControls = new St.BoxLayout({ style_class: 'labels' });
 
         this._selectedIndex = -1;
         this._tabs = [];
     },
 
     appendPage: function(name, child) {
-        let labelBox = new St.BoxLayout({ style_class: "notebook-tab" });
+        let labelBox = new St.BoxLayout({ style_class: 'notebook-tab',
+                                          reactive: true,
+                                          track_hover: true });
         let label = new St.Button({ label: name });
         label.connect('clicked', Lang.bind(this, function () {
             this.selectChild(child);
@@ -84,7 +87,7 @@ Notebook.prototype = {
         if (this._selectedIndex < 0)
             return;
         let tabData = this._tabs[this._selectedIndex];
-        tabData.labelBox.set_style_pseudo_class(null);
+        tabData.labelBox.remove_style_pseudo_class('selected');
         tabData.scrollView.hide();
         this._selectedIndex = -1;
     },
@@ -98,7 +101,7 @@ Notebook.prototype = {
             return;
         }
         let tabData = this._tabs[index];
-        tabData.labelBox.set_style_pseudo_class('selected');
+        tabData.labelBox.add_style_pseudo_class('selected');
         tabData.scrollView.show();
         this._selectedIndex = index;
         this.emit('selection', tabData.child);
@@ -136,9 +139,43 @@ Notebook.prototype = {
         let vAdjust = tabData.scrollView.vscroll.adjustment;
         vAdjust.value = vAdjust.upper - vAdjust.page_size;
     }
-}
+};
 Signals.addSignalMethods(Notebook.prototype);
 
+function objectToString(o) {
+    if (typeof(o) == typeof(objectToString)) {
+        // special case this since the default is way, way too verbose
+        return "<js function>";
+    } else {
+        return "" + o;
+    }
+}
+
+function ObjLink(o, title) {
+    this._init(o, title);
+}
+
+ObjLink.prototype = {
+    __proto__: Link.Link,
+
+    _init: function(o, title) {
+        let text;
+        if (title)
+            text = title;
+        else
+            text = objectToString(o);
+        text = GLib.markup_escape_text(text, -1);
+        this._obj = o;
+        Link.Link.prototype._init.call(this, { label: text });
+        this.actor.get_child().single_line_mode = true;
+        this.actor.connect('clicked', Lang.bind(this, this._onClicked));
+    },
+
+    _onClicked: function (link) {
+        Main.lookingGlass.inspectObject(this._obj, this.actor);
+    }
+};
+
 function Result(command, o, index) {
     this._init(command, o, index);
 }
@@ -148,163 +185,317 @@ Result.prototype = {
         this.index = index;
         this.o = o;
 
-        this.actor = new Big.Box();
+        this.actor = new St.BoxLayout({ vertical: true });
 
         let cmdTxt = new St.Label({ text: command });
-        cmdTxt.ellipsize = Pango.EllipsizeMode.END;
-
-        this.actor.append(cmdTxt, Big.BoxPackFlags.NONE);
-        let resultTxt = new St.Label({ text: "r(" + index + ") = " + o });
-        resultTxt.ellipsize = Pango.EllipsizeMode.END;
-
-        this.actor.append(resultTxt, Big.BoxPackFlags.NONE);
-        let line = new Clutter.Rectangle({ name: "Separator",
-                                           height: 1 });
-        let padBin = new St.Bin({ name: "Separator", x_fill: true, y_fill: true });
+        cmdTxt.clutter_text.ellipsize = Pango.EllipsizeMode.END;
+        this.actor.add(cmdTxt);
+        let box = new St.BoxLayout({});
+        this.actor.add(box);
+        let resultTxt = new St.Label({ text: 'r(' + index + ') = ' });
+        resultTxt.clutter_text.ellipsize = Pango.EllipsizeMode.END;
+        box.add(resultTxt);
+        let objLink = new ObjLink(o);
+        box.add(objLink.actor);
+        let line = new Clutter.Rectangle({ name: 'Separator' });
+        let padBin = new St.Bin({ name: 'Separator', x_fill: true, y_fill: true });
         padBin.add_actor(line);
-        this.actor.append(padBin, Big.BoxPackFlags.NONE);
+        this.actor.add(padBin);
     }
-}
+};
 
-function ActorHierarchy() {
+function WindowList() {
     this._init();
 }
 
-ActorHierarchy.prototype = {
+WindowList.prototype = {
     _init : function () {
-        this._previousTarget = null;
-        this._target = null;
-
-        this._parentList = [];
-
-        this.actor = new St.BoxLayout({ name: "ActorHierarchy", vertical: true });
+        this.actor = new St.BoxLayout({ name: 'Windows', vertical: true, style: 'spacing: 8px' });
+        let display = global.screen.get_display();
+        let tracker = Shell.WindowTracker.get_default();
+        this._updateId = Main.initializeDeferredWork(this.actor, Lang.bind(this, this._updateWindowList));
+        display.connect('window-created', Lang.bind(this, this._updateWindowList));
+        tracker.connect('tracked-windows-changed', Lang.bind(this, this._updateWindowList));
     },
 
-    setTarget: function(actor) {
-        this._previousTarget = this._target;
-        this.target = actor;
-
-        this.actor.get_children().forEach(function (child) { child.destroy(); });
-
-        if (!(actor instanceof Clutter.Actor))
-            return;
-
-        if (this.target == null)
-            return;
-
-        this._parentList = [];
-        let parent = actor;
-        while ((parent = parent.get_parent()) != null) {
-            this._parentList.push(parent);
-
-            let link = new St.Label({ reactive: true,
-                                      text: "" + parent });
-            this.actor.add_actor(link);
-            let parentTarget = parent;
-            link.connect('button-press-event', Lang.bind(this, function () {
-                this._selectByActor(parentTarget);
-                return true;
-            }));
-        }
-        this.emit('selection', actor);
-    },
-
-    _selectByActor: function(actor) {
-        let idx = this._parentList.indexOf(actor);
-        let children = this.actor.get_children();
-        let link = children[idx];
-        this.emit('selection', actor);
-    }
-}
-Signals.addSignalMethods(ActorHierarchy.prototype);
-
-function PropertyInspector() {
-    this._init();
-}
-
-PropertyInspector.prototype = {
-    _init : function () {
-        this._target = null;
-
-        this._parentList = [];
-
-        this.actor = new St.BoxLayout({ name: "PropertyInspector", vertical: true });
-    },
-
-    setTarget: function(actor) {
-        this.target = actor;
-
-        this.actor.get_children().forEach(function (child) { child.destroy(); });
-
-        for (let propName in actor) {
-            let valueStr;
-            try {
-                valueStr = "" + actor[propName];
-            } catch (e) {
-                valueStr = '<error>';
+    _updateWindowList: function() {
+        this.actor.get_children().forEach(function (actor) { actor.destroy(); });
+        let windows = global.get_window_actors();
+        let tracker = Shell.WindowTracker.get_default();
+        for (let i = 0; i < windows.length; i++) {
+            let metaWindow = windows[i].metaWindow;
+            // Avoid multiple connections
+            if (!metaWindow._lookingGlassManaged) {
+                metaWindow.connect('unmanaged', Lang.bind(this, this._updateWindowList));
+                metaWindow._lookingGlassManaged = true;
+            }
+            let box = new St.BoxLayout({ vertical: true });
+            this.actor.add(box);
+            let windowLink = new ObjLink(metaWindow, metaWindow.title);
+            box.add(windowLink.actor, { x_align: St.Align.START, x_fill: false });
+            let propsBox = new St.BoxLayout({ vertical: true, style: 'padding-left: 6px;' });
+            box.add(propsBox);
+            propsBox.add(new St.Label({ text: 'wmclass: ' + metaWindow.get_wm_class() }));
+            let app = tracker.get_window_app(metaWindow);
+            if (app != null && !app.is_transient()) {
+                let icon = app.create_icon_texture(22);
+                let propBox = new St.BoxLayout({ style: 'spacing: 6px; ' });
+                propsBox.add(propBox);
+                propBox.add(new St.Label({ text: 'app: ' }), { y_fill: false });
+                let appLink = new ObjLink(app, app.get_id());
+                propBox.add(appLink.actor, { y_fill: false });
+                propBox.add(icon, { y_fill: false });
+            } else {
+                propsBox.add(new St.Label({ text: '<untracked>' }));
             }
-            let propText = propName + ": " + valueStr;
-            let propDisplay = new St.Label({ reactive: true,
-                                             text: propText });
-            this.actor.add_actor(propDisplay);
         }
     }
+};
+Signals.addSignalMethods(WindowList.prototype);
+
+function ObjInspector() {
+    this._init();
 }
 
+ObjInspector.prototype = {
+    _init : function () {
+        this._obj = null;
+        this._previousObj = null;
+
+        this._parentList = [];
+
+        this.actor = new St.ScrollView({ x_fill: true, y_fill: true });
+        this.actor.get_hscroll_bar().hide();
+        this._container = new St.BoxLayout({ name: 'LookingGlassPropertyInspector',
+                                             style_class: 'lg-dialog',
+                                             vertical: true });
+        this.actor.add_actor(this._container);
+    },
+
+    selectObject: function(obj, skipPrevious) {
+        if (!skipPrevious)
+            this._previousObj = this._obj;
+        else
+            this._previousObj = null;
+        this._obj = obj;
+
+        this._container.get_children().forEach(function (child) { child.destroy(); });
+
+        let hbox = new St.BoxLayout({ style_class: 'lg-obj-inspector-title' });
+        this._container.add_actor(hbox);
+        let label = new St.Label({ text: 'Inspecting: %s: %s'.format(typeof(obj),
+                                                                     objectToString(obj)) });
+        label.single_line_mode = true;
+        hbox.add(label, { expand: true, y_fill: false });
+        let button = new St.Button({ label: 'Insert', style_class: 'lg-obj-inspector-button' });
+        button.connect('clicked', Lang.bind(this, this._onInsert));
+        hbox.add(button);
+
+        if (this._previousObj != null) {
+            button = new St.Button({ label: 'Back', style_class: 'lg-obj-inspector-button' });
+            button.connect('clicked', Lang.bind(this, this._onBack));
+            hbox.add(button);
+        }
+
+        button = new St.Button({ style_class: 'window-close' });
+        button.connect('clicked', Lang.bind(this, this.close));
+        hbox.add(button);
+        if (typeof(obj) == typeof({})) {
+            for (let propName in obj) {
+                let valueStr;
+                let link;
+                try {
+                    let prop = obj[propName];
+                    link = new ObjLink(prop).actor;
+                } catch (e) {
+                    link = new St.Label({ text: '<error>' });
+                }
+                let hbox = new St.BoxLayout();
+                let propText = propName + ": " + valueStr;
+                hbox.add(new St.Label({ text: propName + ': ' }));
+                hbox.add(link);
+                this._container.add_actor(hbox);
+            }
+        }
+    },
+
+    open: function(sourceActor) {
+        if (this._open)
+            return;
+        this._previousObj = null;
+        this._open = true;
+        this.actor.show();
+        if (sourceActor) {
+            this.actor.set_scale(0, 0);
+            let [sourceX, sourceY] = sourceActor.get_transformed_position();
+            let [sourceWidth, sourceHeight] = sourceActor.get_transformed_size();
+            this.actor.move_anchor_point(Math.floor(sourceX + sourceWidth / 2),
+                                         Math.floor(sourceY + sourceHeight / 2));
+            Tweener.addTween(this.actor, { scale_x: 1, scale_y: 1,
+                                           transition: "easeOutQuad",
+                                           time: 0.2 });
+        } else {
+            this.actor.set_scale(1, 1);
+        }
+    },
+
+    close: function() {
+        if (!this._open)
+            return;
+        this._open = false;
+        this.actor.hide();
+        this._previousObj = null;
+        this._obj = null;
+    },
+
+    _onInsert: function() {
+        let obj = this._obj;
+        this.close();
+        Main.lookingGlass.insertObject(obj);
+    },
+
+    _onBack: function() {
+        this.selectObject(this._previousObj, true);
+    }
+};
+
 function Inspector() {
     this._init();
 }
 
 Inspector.prototype = {
     _init: function() {
-        let width = 150;
-        let primary = global.get_primary_monitor();
-        let eventHandler = new St.BoxLayout({ name: "LookingGlassDialog",
+        let container = new Shell.GenericContainer({ width: 0,
+                                                     height: 0 });
+        container.connect('allocate', Lang.bind(this, this._allocate));
+        Main.uiGroup.add_actor(container);
+
+        let eventHandler = new St.BoxLayout({ name: 'LookingGlassDialog',
                                               vertical: false,
-                                              y: primary.y + Math.floor(primary.height / 2),
                                               reactive: true });
-        eventHandler.connect('notify::allocation', Lang.bind(this, function () {
-            eventHandler.x = primary.x + Math.floor((primary.width - eventHandler.width) / 2);
-        }));
-        global.stage.add_actor(eventHandler);
-        let displayText = new St.Label();
-        eventHandler.add(displayText, { expand: true });
+        this._eventHandler = eventHandler;
+        container.add_actor(eventHandler);
+        this._displayText = new St.Label();
+        eventHandler.add(this._displayText, { expand: true });
 
-        let borderPaintTarget = null;
-        let borderPaintId = null;
-        eventHandler.connect('destroy', Lang.bind(this, function() {
-            if (borderPaintTarget != null)
-                borderPaintTarget.disconnect(borderPaintId);
-        }));
-
-        eventHandler.connect('button-press-event', Lang.bind(this, function (actor, event) {
-            Clutter.ungrab_pointer(eventHandler);
-
-            let [stageX, stageY] = event.get_coords();
-            let target = global.stage.get_actor_at_pos(Clutter.PickMode.ALL,
-                                                       stageX,
-                                                       stageY);
-            this.emit('target', target, stageX, stageY);
-            eventHandler.destroy();
-            this.emit('closed');
-            return true;
-        }));
-
-        eventHandler.connect('motion-event', Lang.bind(this, function (actor, event) {
-            let [stageX, stageY] = event.get_coords();
-            let target = global.stage.get_actor_at_pos(Clutter.PickMode.ALL,
-                                                       stageX,
-                                                       stageY);
-            displayText.text = '<inspect x: ' + stageX + ' y: ' + stageY + '> ' + target;
-            if (borderPaintTarget != null)
-                borderPaintTarget.disconnect(borderPaintId);
-            borderPaintTarget = target;
-            borderPaintId = Shell.add_hook_paint_red_border(target);
-            return true;
-        }));
+        this._borderPaintTarget = null;
+        this._borderPaintId = null;
+        eventHandler.connect('destroy', Lang.bind(this, this._onDestroy));
+        eventHandler.connect('key-press-event', Lang.bind(this, this._onKeyPressEvent));
+        eventHandler.connect('button-press-event', Lang.bind(this, this._onButtonPressEvent));
+        eventHandler.connect('scroll-event', Lang.bind(this, this._onScrollEvent));
+        eventHandler.connect('motion-event', Lang.bind(this, this._onMotionEvent));
         Clutter.grab_pointer(eventHandler);
+        Clutter.grab_keyboard(eventHandler);
+
+        // this._target is the actor currently shown by the inspector.
+        // this._pointerTarget is the actor directly under the pointer.
+        // Normally these are the same, but if you use the scroll wheel
+        // to drill down, they'll diverge until you either scroll back
+        // out, or move the pointer outside of _pointerTarget.
+        this._target = null;
+        this._pointerTarget = null;
+    },
+
+    _allocate: function(actor, box, flags) {
+        let primary = global.get_primary_monitor();
+
+        let [minWidth, minHeight, natWidth, natHeight] =
+            this._eventHandler.get_preferred_size();
+
+        let childBox = new Clutter.ActorBox();
+        childBox.x1 = primary.x + Math.floor((primary.width - natWidth) / 2);
+        childBox.x2 = childBox.x1 + natWidth;
+        childBox.y1 = primary.y + Math.floor((primary.height - natHeight) / 2);
+        childBox.y2 = childBox.y1 + natHeight;
+        this._eventHandler.allocate(childBox, flags);
+    },
+
+    _close: function() {
+        Clutter.ungrab_pointer(this._eventHandler);
+        Clutter.ungrab_keyboard(this._eventHandler);
+        this._eventHandler.destroy();
+        this.emit('closed');
+    },
+
+    _onDestroy: function() {
+        if (this._borderPaintTarget != null)
+            this._borderPaintTarget.disconnect(this._borderPaintId);
+    },
+
+    _onKeyPressEvent: function (actor, event) {
+        if (event.get_key_symbol() == Clutter.Escape)
+            this._close();
+        return true;
+    },
+
+    _onButtonPressEvent: function (actor, event) {
+        if (this._target) {
+            let [stageX, stageY] = event.get_coords();
+            this.emit('target', this._target, stageX, stageY);
+        }
+        this._close();
+        return true;
+    },
+
+    _onScrollEvent: function (actor, event) {
+        switch (event.get_scroll_direction()) {
+        case Clutter.ScrollDirection.UP:
+            // select parent
+            let parent = this._target.get_parent();
+            if (parent != null) {
+                this._target = parent;
+                this._update(event);
+            }
+            break;
+
+        case Clutter.ScrollDirection.DOWN:
+            // select child
+            if (this._target != this._pointerTarget) {
+                let child = this._pointerTarget;
+                while (child) {
+                    let parent = child.get_parent();
+                    if (parent == this._target)
+                        break;
+                    child = parent;
+                }
+                if (child) {
+                    this._target = child;
+                    this._update(event);
+                }
+            }
+            break;
+
+        default:
+            break;
+        }
+        return true;
+    },
+
+    _onMotionEvent: function (actor, event) {
+        this._update(event);
+        return true;
+    },
+
+    _update: function(event) {
+        let [stageX, stageY] = event.get_coords();
+        let target = global.stage.get_actor_at_pos(Clutter.PickMode.ALL,
+                                                   stageX,
+                                                   stageY);
+
+        if (target != this._pointerTarget)
+            this._target = target;
+        this._pointerTarget = target;
+
+        let position = '[inspect x: ' + stageX + ' y: ' + stageY + ']';
+        this._displayText.text = '';
+        this._displayText.text = position + ' ' + this._target;
+        if (this._borderPaintTarget != null)
+            this._borderPaintTarget.disconnect(this._borderPaintId);
+        this._borderPaintTarget = this._target;
+        this._borderPaintId = Shell.add_hook_paint_red_border(this._target);
     }
-}
+};
 
 Signals.addSignalMethods(Inspector.prototype);
 
@@ -317,18 +508,23 @@ ErrorLog.prototype = {
         this.actor = new St.BoxLayout();
         this.text = new St.Label();
         this.actor.add(this.text);
+        // We need to override StLabel's default ellipsization when
+        // using line_wrap; otherwise ClutterText's layout is going
+        // to constrain both the width and height, which prevents
+        // scrolling.
+        this.text.clutter_text.ellipsize = Pango.EllipsizeMode.NONE;
         this.text.clutter_text.line_wrap = true;
         this.actor.connect('notify::mapped', Lang.bind(this, this._renderText));
     },
 
     _formatTime: function(d){
-        function pad(n) { return n < 10 ? '0' + n : n };
+        function pad(n) { return n < 10 ? '0' + n : n; }
         return d.getUTCFullYear()+'-'
             + pad(d.getUTCMonth()+1)+'-'
             + pad(d.getUTCDate())+'T'
             + pad(d.getUTCHours())+':'
             + pad(d.getUTCMinutes())+':'
-            + pad(d.getUTCSeconds())+'Z'
+            + pad(d.getUTCSeconds())+'Z';
     },
 
     _renderText: function() {
@@ -338,11 +534,11 @@ ErrorLog.prototype = {
         let stack = Main._getAndClearErrorStack();
         for (let i = 0; i < stack.length; i++) {
             let logItem = stack[i];
-            text += logItem.category + " t=" + this._formatTime(new Date(logItem.timestamp)) + " " + logItem.message + "\n";
+            text += logItem.category + ' t=' + this._formatTime(new Date(logItem.timestamp)) + ' ' + logItem.message + '\n';
         }
         this.text.text = text;
     }
-}
+};
 
 function Extensions() {
     this._init();
@@ -398,7 +594,7 @@ Extensions.prototype = {
             case ExtensionSystem.ExtensionState.OUT_OF_DATE:
                 return _("Out of date");
         }
-        return "Unknown"; // Not translated, shouldn't appear
+        return 'Unknown'; // Not translated, shouldn't appear
     },
 
     _createExtensionDisplay: function(meta) {
@@ -444,7 +640,7 @@ function LookingGlass() {
 LookingGlass.prototype = {
     _init : function() {
         this._idleHistorySaveId = 0;
-        let historyPath = global.configdir + "/lookingglass-history.txt";
+        let historyPath = global.userdatadir + '/lookingglass-history.txt';
         this._historyFile = Gio.file_new_for_path(historyPath);
         this._savedText = null;
         this._historyNavIndex = -1;
@@ -463,22 +659,28 @@ LookingGlass.prototype = {
         // Sort of magic, but...eh.
         this._maxItems = 150;
 
-        this.actor = new St.BoxLayout({ name: "LookingGlassDialog",
+        this.actor = new St.BoxLayout({ name: 'LookingGlassDialog',
+                                        style_class: 'lg-dialog',
                                         vertical: true,
                                         visible: false });
 
-        let gconf = Shell.GConf.get_default();
-        gconf.watch_directory("/desktop/gnome/interface");
-        gconf.connect("changed::/desktop/gnome/interface/monospace_font_name",
-                      Lang.bind(this, this._updateFont));
+        let gconf = GConf.Client.get_default();
+        gconf.add_dir('/desktop/gnome/interface', GConf.ClientPreloadType.PRELOAD_NONE);
+        gconf.notify_add('/desktop/gnome/interface/monospace_font_name',
+                         Lang.bind(this, this._updateFont));
         this._updateFont();
 
-        global.stage.add_actor(this.actor);
+        Main.uiGroup.add_actor(this.actor);
 
-        let toolbar = new St.BoxLayout({ name: "Toolbar" });
+        this._objInspector = new ObjInspector();
+        Main.uiGroup.add_actor(this._objInspector.actor);
+        this._objInspector.actor.hide();
+
+        let toolbar = new St.BoxLayout({ name: 'Toolbar' });
         this.actor.add_actor(toolbar);
-        let inspectIcon = Shell.TextureCache.get_default().load_gicon(new Gio.ThemedIcon({ name: 'gtk-color-picker' }),
-                                                                      24);
+        let inspectIcon = St.TextureCache.get_default().load_icon_name('gtk-color-picker',
+                                                                       St.IconType.SYMBOLIC,
+                                                                       24);
         toolbar.add_actor(inspectIcon);
         inspectIcon.reactive = true;
         inspectIcon.connect('button-press-event', Lang.bind(this, function () {
@@ -486,7 +688,6 @@ LookingGlass.prototype = {
             inspector.connect('target', Lang.bind(this, function(i, target, stageX, stageY) {
                 this._pushResult('<inspect x:' + stageX + ' y:' + stageY + '>',
                                  target);
-                this._hierarchy.setTarget(target);
             }));
             inspector.connect('closed', Lang.bind(this, function() {
                 this.actor.show();
@@ -504,18 +705,17 @@ LookingGlass.prototype = {
         toolbar.add(emptyBox, { expand: true });
         toolbar.add_actor(notebook.tabControls);
 
-        this._evalBox = new St.BoxLayout({ name: "EvalBox", vertical: true });
+        this._evalBox = new St.BoxLayout({ name: 'EvalBox', vertical: true });
         notebook.appendPage('Evaluator', this._evalBox);
 
-        this._resultsArea = new Big.Box({ orientation: Big.BoxOrientation.VERTICAL,
-                                          spacing: 4 });
+        this._resultsArea = new St.BoxLayout({ name: 'ResultsArea', vertical: true });
         this._evalBox.add(this._resultsArea, { expand: true });
 
-        let entryArea = new Big.Box({ orientation: Big.BoxOrientation.HORIZONTAL });
+        let entryArea = new St.BoxLayout({ name: 'EntryArea' });
         this._evalBox.add_actor(entryArea);
 
         let label = new St.Label({ text: 'js>>> ' });
-        entryArea.append(label, Big.BoxPackFlags.NONE);
+        entryArea.add(label);
 
         this._entry = new St.Entry();
         /* unmapping the edit box will un-focus it, undo that */
@@ -523,17 +723,14 @@ LookingGlass.prototype = {
             if (child == this._evalBox)
                 global.stage.set_key_focus(this._entry);
         }));
-        entryArea.append(this._entry, Big.BoxPackFlags.EXPAND);
+        entryArea.add(this._entry, { expand: true });
 
-        this._hierarchy = new ActorHierarchy();
-        notebook.appendPage('Hierarchy', this._hierarchy.actor);
-
-        this._propInspector = new PropertyInspector();
-        notebook.appendPage('Properties', this._propInspector.actor);
-        this._hierarchy.connect('selection', Lang.bind(this, function (h, actor) {
-            this._pushResult('<parent selection>', actor);
+        this._windowList = new WindowList();
+        this._windowList.connect('selected', Lang.bind(this, function(list, window) {
             notebook.selectIndex(0);
+            this._pushResult('<window selection>', window);
         }));
+        notebook.appendPage('Windows', this._windowList.actor);
 
         this._errorLog = new ErrorLog();
         notebook.appendPage('Errors', this._errorLog.actor);
@@ -547,7 +744,7 @@ LookingGlass.prototype = {
             // newline-separated.
             text.replace('\n', ' ');
             // Strip leading and trailing whitespace
-            text = text.replace(/^\s+/g, "").replace(/\s+$/g, "");
+            text = text.replace(/^\s+/g, '').replace(/\s+$/g, '');
             if (text == '')
                 return true;
             this._evaluate(text);
@@ -582,11 +779,11 @@ LookingGlass.prototype = {
     },
 
     _updateFont: function() {
-        let gconf = Shell.GConf.get_default();
-        let fontName = gconf.get_string("/desktop/gnome/interface/monospace_font_name");
+        let gconf = GConf.Client.get_default();
+        let fontName = gconf.get_string('/desktop/gnome/interface/monospace_font_name');
         // This is mishandled by the scanner - should by Pango.FontDescription_from_string(fontName);
         // https://bugzilla.gnome.org/show_bug.cgi?id=595889
-        let fontDesc = Pango.Font.description_from_string(fontName);
+        let fontDesc = Pango.font_description_from_string(fontName);
         // We ignore everything but size and style; you'd be crazy to set your system-wide
         // monospace font to be bold/oblique/etc. Could easily be added here.
         this.actor.style =
@@ -621,8 +818,7 @@ LookingGlass.prototype = {
         let index = this._results.length + this._offset;
         let result = new Result('>>> ' + command, obj, index);
         this._results.push(result);
-        this._resultsArea.append(result.actor, Big.BoxPackFlags.NONE);
-        this._propInspector.setTarget(obj);
+        this._resultsArea.add(result.actor);
         if (this._borderPaintTarget != null) {
             this._borderPaintTarget.disconnect(this._borderPaintId);
             this._borderPaintTarget = null;
@@ -657,11 +853,10 @@ LookingGlass.prototype = {
         try {
             resultObj = eval(fullCmd);
         } catch (e) {
-            resultObj = "<exception " + e + ">";
+            resultObj = '<exception ' + e + '>';
         }
 
         this._pushResult(command, resultObj);
-        this._hierarchy.setTarget(null);
         this._entry.text = '';
     },
 
@@ -691,6 +886,9 @@ LookingGlass.prototype = {
         this.actor.y = this._hiddenY;
         this.actor.width = myWidth;
         this.actor.height = myHeight;
+        this._objInspector.actor.set_size(Math.floor(myWidth * 0.8), Math.floor(myHeight * 0.8));
+        this._objInspector.actor.set_position(this.actor.x + Math.floor(myWidth * 0.1),
+                                              this._targetY + Math.floor(myHeight * 0.1));
     },
 
     slaveTo: function(actor) {
@@ -701,11 +899,24 @@ LookingGlass.prototype = {
         this._resizeTo(actor);
     },
 
+    insertObject: function(obj) {
+        this._pushResult('<insert>', obj);
+    },
+
+    inspectObject: function(obj, sourceActor) {
+        this._objInspector.open(sourceActor);
+        this._objInspector.selectObject(obj);
+    },
+
     // Handle key events which are relevant for all tabs of the LookingGlass
     _globalKeyPressEvent : function(actor, event) {
         let symbol = event.get_key_symbol();
         if (symbol == Clutter.Escape) {
-            this.close();
+            if (this._objInspector.actor.visible) {
+                this._objInspector.close();
+            } else {
+                this.close();
+            }
             return true;
         }
         return false;
@@ -730,7 +941,7 @@ LookingGlass.prototype = {
         global.stage.set_key_focus(this._entry);
 
         Tweener.addTween(this.actor, { time: 0.5,
-                                       transition: "easeOutQuad",
+                                       transition: 'easeOutQuad',
                                        y: this._targetY
                                      });
     },
@@ -742,6 +953,8 @@ LookingGlass.prototype = {
         if (this._keyPressEventId)
             global.stage.disconnect(this._keyPressEventId);
 
+        this._objInspector.actor.hide();
+
         this._historyNavIndex = -1;
         this._open = false;
         Tweener.removeTweens(this.actor);
@@ -755,7 +968,7 @@ LookingGlass.prototype = {
         Main.popModal(this.actor);
 
         Tweener.addTween(this.actor, { time: 0.5,
-                                       transition: "easeOutQuad",
+                                       transition: 'easeOutQuad',
                                        y: this._hiddenY,
                                        onComplete: Lang.bind(this, function () {
                                            this.actor.hide();
