@@ -12,6 +12,7 @@ const St = imports.gi.St;
 const Tweener = imports.ui.tweener;
 
 const Main = imports.ui.main;
+const BoxPointer = imports.ui.boxpointer;
 const Params = imports.misc.params;
 
 const ANIMATION_TIME = 0.2;
@@ -116,11 +117,6 @@ Notification.prototype = {
         this._capturedEventId = 0;
         this._keyPressId = 0;
 
-        source.connect('clicked', Lang.bind(this,
-            function() {
-                this.emit('dismissed');
-            }));
-
         source.connect('destroy', Lang.bind(this, this.destroy));
 
         this.actor = new St.Table({ name: 'notification',
@@ -130,7 +126,7 @@ Notification.prototype = {
             function (actor, event) {
                 if (!this._actionArea ||
                     !this._actionArea.contains(event.get_source()))
-                    this.source.clicked();
+                    this.emit('clicked');
             }));
 
         // The first line should have the title, followed by the
@@ -309,6 +305,8 @@ Notification.prototype = {
             this._actionArea = null;
             if (this._buttonBox)
                 this._buttonBox = null;
+        } else {
+            this._addBannerBody();
         }
         this._actionArea = actor;
 
@@ -362,8 +360,7 @@ Notification.prototype = {
     },
 
     _styleChanged: function() {
-        let [hasSpacing, spacing] = this.actor.get_theme_node().get_length('spacing-columns', false);
-        this._spacing = hasSpacing ? spacing : 0;
+        this._spacing = this.actor.get_theme_node().get_length('spacing-columns');
     },
 
     _bannerBoxGetPreferredWidth: function(actor, forHeight, alloc) {
@@ -663,26 +660,26 @@ Source.prototype = {
     },
 
     notify: function(notification) {
-        if (this.notification)
+        if (this.notification) {
+            this.notification.disconnect(this._notificationClickedId);
             this.notification.disconnect(this._notificationDestroyedId);
+        }
 
         this.notification = notification;
 
+        this._notificationClickedId = notification.connect('clicked', Lang.bind(this, this._notificationClicked));
         this._notificationDestroyedId = notification.connect('destroy', Lang.bind(this,
             function () {
                 if (this.notification == notification) {
                     this.notification = null;
                     this._notificationDestroyedId = 0;
+                    this._notificationClickedId = 0;
                 }
             }));
 
         this.emit('notify', notification);
     },
 
-    clicked: function() {
-        this.emit('clicked');
-    },
-
     destroy: function() {
         this.emit('destroy');
     },
@@ -694,145 +691,64 @@ Source.prototype = {
         if (this._iconBin.child)
             this._iconBin.child.destroy();
         this._iconBin.child = icon;
+    },
+
+    // Default implementation is to do nothing, but subclass can override
+    _notificationClicked: function(notification) {
     }
 };
 Signals.addSignalMethods(Source.prototype);
 
-function SummaryItem(source, minTitleWidth) {
-    this._init(source, minTitleWidth);
+function SummaryItem(source) {
+    this._init(source);
 }
 
 SummaryItem.prototype = {
-    _init: function(source, minTitleWidth) {
+    _init: function(source) {
         this.source = source;
-        // The message tray items should all be the same width when expanded. Because the only variation is introduced by the width of the title,
-        // we pass in the desired minimum title width, which is the maximum title width of the items which are currently in the tray. If the width
-        // of the title of this item is greater (up to MAX_SOURCE_TITLE_WIDTH), then that width will be used, and the width of all the other items
-        // in the message tray will be readjusted.
-        this._minTitleWidth = minTitleWidth;
         this.actor = new St.Button({ style_class: 'summary-source-button',
                                      reactive: true,
                                      track_hover: true });
 
-        this._sourceBox = new  Shell.GenericContainer({ style_class: 'summary-source',
-                                                        reactive: true });
-        this._sourceBox.connect('get-preferred-width', Lang.bind(this, this._getPreferredWidth));
-        this._sourceBox.connect('get-preferred-height', Lang.bind(this, this._getPreferredHeight));
-        this._sourceBox.connect('allocate', Lang.bind(this, this._allocate));
+        this._sourceBox = new St.BoxLayout({ style_class: 'summary-source' });
 
         this._sourceIcon = source.getSummaryIcon();
-        this._sourceTitleBin = new St.Bin({ y_align: St.Align.MIDDLE, x_fill: true });
+        this._sourceTitleBin = new St.Bin({ y_align: St.Align.MIDDLE,
+                                            x_fill: true,
+                                            clip_to_allocation: true });
         this._sourceTitle = new St.Label({ style_class: 'source-title',
                                            text: source.title });
         this._sourceTitle.clutter_text.ellipsize = Pango.EllipsizeMode.NONE;
         this._sourceTitleBin.child = this._sourceTitle;
+        this._sourceTitleBin.width = 0;
 
         this._sourceBox.add_actor(this._sourceIcon);
-        this._sourceBox.add_actor(this._sourceTitleBin);
-        this._widthFraction = 0;
+        this._sourceBox.add_actor(this._sourceTitleBin, { expand: true });
         this.actor.child = this._sourceBox;
     },
 
+    // getTitleNaturalWidth, getTitleWidth, and setTitleWidth include
+    // the spacing between the icon and title (which is actually
+    // _sourceTitle's padding-left) as part of the width.
+
     getTitleNaturalWidth: function() {
-        let [sourceTitleBinMinWidth, sourceTitleBinNaturalWidth] =
-            this._sourceTitleBin.get_preferred_width(-1);
-        return Math.min(sourceTitleBinNaturalWidth, MAX_SOURCE_TITLE_WIDTH);
+        let [minWidth, naturalWidth] = this._sourceTitle.get_preferred_width(-1);
+
+        return Math.min(naturalWidth, MAX_SOURCE_TITLE_WIDTH);
     },
 
-    setMinTitleWidth: function(minTitleWidth) {
-        this._minTitleWidth = minTitleWidth;
+    getTitleWidth: function() {
+        return this._sourceTitleBin.width;
     },
 
-    _getPreferredWidth: function(actor, forHeight, alloc) {
-        let [found, spacing] = this._sourceBox.get_theme_node().get_length('spacing', false);
-        if (!found)
-            spacing = 0;
-        let [sourceIconMinWidth, sourceIconNaturalWidth] = this._sourceIcon.get_preferred_width(forHeight);
-        let [sourceTitleBinMinWidth, sourceTitleBinNaturalWidth] =
-            this._sourceTitleBin.get_preferred_width(forHeight);
-        let minWidth = sourceIconNaturalWidth +
-                       (this._widthFraction > 0 ? spacing : 0) +
-                       this._widthFraction * Math.min(Math.max(sourceTitleBinNaturalWidth, this._minTitleWidth),
-                                                      MAX_SOURCE_TITLE_WIDTH);
-        alloc.min_size = minWidth;
-        alloc.natural_size = minWidth;
+    setTitleWidth: function(width) {
+        width = Math.round(width);
+        if (width != this._sourceTitleBin.width)
+            this._sourceTitleBin.width = width;
     },
 
-    _getPreferredHeight: function(actor, forWidth, alloc) {
-        let [sourceIconMinHeight, sourceIconNaturalHeight] = this._sourceIcon.get_preferred_height(forWidth);
-        alloc.min_size = sourceIconNaturalHeight;
-        alloc.natural_size = sourceIconNaturalHeight;
-    },
-
-    _allocate: function (actor, box, flags) {
-        let width = box.x2 - box.x1;
-        let height = box.y2 - box.y1;
-
-        let [sourceIconMinWidth, sourceIconNaturalWidth] = this._sourceIcon.get_preferred_width(-1);
-        let [sourceIconMinHeight, sourceIconNaturalHeight] = this._sourceIcon.get_preferred_height(-1);
-
-        let iconBox = new Clutter.ActorBox();
-        iconBox.x1 = 0;
-        iconBox.y1 = 0;
-        iconBox.x2 = sourceIconNaturalWidth;
-        iconBox.y2 = sourceIconNaturalHeight;
-
-        this._sourceIcon.allocate(iconBox, flags);
-
-        let [found, spacing] = this._sourceBox.get_theme_node().get_length('spacing', false);
-        if (!found)
-            spacing = 0;
-
-        let titleBox = new Clutter.ActorBox();
-        if (width > sourceIconNaturalWidth + spacing) {
-            titleBox.x1 = iconBox.x2 + spacing;
-            titleBox.x2 = width;
-        } else {
-            titleBox.x1 = iconBox.x2;
-            titleBox.x2 = iconBox.x2;
-        }
-        titleBox.y1 = 0;
-        titleBox.y2 = height;
-
-        this._sourceTitleBin.allocate(titleBox, flags);
-
-        this._sourceTitleBin.set_clip(0, 0, titleBox.x2 - titleBox.x1, height);
-    },
-
-    expand: function() {
-        // this._adjustEllipsization replaces some text with the dots at the end of the animation,
-        // and then we replace the dots with the text before we begin the animation to collapse
-        // the title. These changes are not noticeable at the speed with which we do the animation,
-        // while animating in the ellipsized mode does not look good.
-        Tweener.addTween(this,
-                         { widthFraction: 1,
-                           time: ANIMATION_TIME,
-                           transition: 'linear',
-                           onComplete: this._adjustEllipsization,
-                           onCompleteScope: this });
-    },
-
-    collapse: function() {
-        this._sourceTitle.clutter_text.ellipsize = Pango.EllipsizeMode.NONE;
-        Tweener.addTween(this,
-                         { widthFraction: 0,
-                           time: ANIMATION_TIME,
-                           transition: 'linear' });
-    },
-
-    _adjustEllipsization: function() {
-        let [sourceTitleBinMinWidth, sourceTitleBinNaturalWidth] = this._sourceTitleBin.get_preferred_width(-1);
-        if (sourceTitleBinNaturalWidth > MAX_SOURCE_TITLE_WIDTH)
-            this._sourceTitle.clutter_text.ellipsize = Pango.EllipsizeMode.END;
-    },
-
-    set widthFraction(widthFraction) {
-        this._widthFraction = widthFraction;
-        this._sourceBox.queue_relayout();
-    },
-
-    get widthFraction() {
-        return this._widthFraction;
+    setEllipsization: function(mode) {
+        this._sourceTitle.clutter_text.ellipsize = mode;
     }
 };
 
@@ -862,15 +778,27 @@ MessageTray.prototype = {
         this._summaryBin.child = this._summary;
         this._summaryBin.opacity = 0;
 
-        this._summaryNotificationBin = new St.Bin({ name: 'summary-notification-bin',
-                                                    anchor_gravity: Clutter.Gravity.NORTH_EAST,
-                                                    reactive: true,
-                                                    track_hover: true });
-        this.actor.add_actor(this._summaryNotificationBin);
-        this._summaryNotificationBin.lower_bottom();
-        this._summaryNotificationBin.hide();
+        this._summaryMotionId = 0;
+
+        this._summaryNotificationBoxPointer = new BoxPointer.BoxPointer(St.Side.BOTTOM,
+                                                                        { reactive: true,
+                                                                          track_hover: true });
+        this._summaryNotificationBoxPointer.actor.style_class = 'summary-notification-boxpointer';
+        this.actor.add_actor(this._summaryNotificationBoxPointer.actor);
+        this._summaryNotificationBoxPointer.actor.lower_bottom();
+        this._summaryNotificationBoxPointer.actor.hide();
+
         this._summaryNotification = null;
         this._clickedSummaryItem = null;
+        this._clickedSummaryItemAllocationChangedId = 0;
+        this._expandedSummaryItem = null;
+        this._summaryItemTitleWidth = 0;
+
+        // To simplify the summary item animation code, we pretend
+        // that there's an invisible SummaryItem to the left of the
+        // leftmost real summary item, and that it's expanded when all
+        // of the other items are collapsed.
+        this._imaginarySummaryItemTitleWidth = 0;
 
         this._trayState = State.HIDDEN;
         this._locked = false;
@@ -893,7 +821,7 @@ MessageTray.prototype = {
         Main.chrome.addActor(this.actor, { affectsStruts: false,
                                            visibleInOverview: true });
         Main.chrome.trackActor(this._notificationBin);
-        Main.chrome.trackActor(this._summaryNotificationBin);
+        Main.chrome.trackActor(this._summaryNotificationBoxPointer.actor);
 
         global.gdk_screen.connect('monitors-changed', Lang.bind(this, this._setSizePosition));
 
@@ -917,6 +845,11 @@ MessageTray.prototype = {
             }));
 
         this._summaryItems = [];
+        // We keep a list of new summary items that were added to the summary since the last
+        // time it was shown to the user. We automatically show the summary to the user if there
+        // are items in this list once the notifications are done showing or once an item gets
+        // added to the summary without a notification being shown.
+        this._newSummaryItems = [];
         this._longestSummaryItem = null;
     },
 
@@ -930,7 +863,6 @@ MessageTray.prototype = {
 
         // These work because of their anchor_gravity
         this._summaryBin.x = primary.width;
-        this._summaryNotificationBin.x = primary.width;
     },
 
     contains: function(source) {
@@ -951,22 +883,20 @@ MessageTray.prototype = {
             return;
         }
 
-        let minTitleWidth = (this._longestSummaryItem ? this._longestSummaryItem.getTitleNaturalWidth() : 0);
-        let summaryItem = new SummaryItem(source, minTitleWidth);
+        let summaryItem = new SummaryItem(source);
 
         this._summary.insert_actor(summaryItem.actor, 0);
-        this._summaryNeedsToBeShown = true;
 
-        let newItemTitleWidth = summaryItem.getTitleNaturalWidth();
-        if (newItemTitleWidth > minTitleWidth) {
-            for (let i = 0; i < this._summaryItems.length; i++) {
-                this._summaryItems[i].setMinTitleWidth(newItemTitleWidth);
-            }
-            summaryItem.setMinTitleWidth(newItemTitleWidth);
+        let titleWidth = summaryItem.getTitleNaturalWidth();
+        if (titleWidth > this._summaryItemTitleWidth) {
+            this._summaryItemTitleWidth = titleWidth;
+            if (!this._expandedSummaryItem)
+                this._imaginarySummaryItemTitleWidth = titleWidth;
             this._longestSummaryItem = summaryItem;
         }
 
         this._summaryItems.push(summaryItem);
+        this._newSummaryItems.push(summaryItem);
 
         source.connect('notify', Lang.bind(this, this._onNotify));
 
@@ -1007,21 +937,26 @@ MessageTray.prototype = {
 
         this._summary.remove_actor(this._summaryItems[index].actor);
 
+        let newSummaryItemsIndex = this._newSummaryItems.indexOf(this._summaryItems[index]);
+        if (newSummaryItemsIndex != -1)
+            this._newSummaryItems.splice(newSummaryItemsIndex, 1);
+
         this._summaryItems.splice(index, 1);
         if (this._longestSummaryItem.source == source) {
-
-            let maxTitleWidth = 0;
+            let newTitleWidth = 0;
             this._longestSummaryItem = null;
             for (let i = 0; i < this._summaryItems.length; i++) {
                 let summaryItem = this._summaryItems[i];
-                if (summaryItem.getTitleNaturalWidth() > maxTitleWidth) {
-                    maxTitleWidth = summaryItem.getTitleNaturalWidth();
+                let titleWidth = summaryItem.getTitleNaturalWidth();
+                if (titleWidth > newTitleWidth) {
+                    newTitleWidth = titleWidth;
                     this._longestSummaryItem = summaryItem;
                 }
             }
-            for (let i = 0; i < this._summaryItems.length; i++) {
-                this._summaryItems[i].setMinTitleWidth(maxTitleWidth);
-            }
+
+            this._summaryItemTitleWidth = newTitleWidth;
+            if (!this._expandedSummaryItem)
+                this._imaginarySummaryItemTitleWidth = newTitleWidth;
         }
 
         let needUpdate = false;
@@ -1032,7 +967,7 @@ MessageTray.prototype = {
             needUpdate = true;
         }
         if (this._clickedSummaryItem && this._clickedSummaryItem.source == source) {
-            this._clickedSummaryItem = null;
+            this._unsetClickedSummaryItem();
             needUpdate = true;
         }
 
@@ -1061,7 +996,7 @@ MessageTray.prototype = {
         if (!this._locked)
             return;
         this._locked = false;
-        this._clickedSummaryItem = null;
+        this._unsetClickedSummaryItem();
         this._updateState();
     },
 
@@ -1097,17 +1032,94 @@ MessageTray.prototype = {
     },
 
     _onSummaryItemHoverChanged: function(summaryItem) {
-        if (summaryItem.actor.hover)
-            summaryItem.expand();
+        // We can't just animate individual summary items as the
+        // pointer moves in and out of them, because if they don't
+        // move in sync you get weird-looking wobbling. So whenever
+        // there's a change, we have to re-tween the entire summary
+        // area.
+
+        if (summaryItem.actor.hover) {
+            if (summaryItem == this._expandedSummaryItem)
+                return;
+
+            this._expandedSummaryItem = summaryItem;
+        } else {
+            if (summaryItem != this._expandedSummaryItem)
+                return;
+
+            this._expandedSummaryItem = null;
+
+            // Turn off ellipsization while collapsing; it looks better
+            summaryItem.setEllipsization(Pango.EllipsizeMode.NONE);
+        }
+
+        // We tween on a "_expandedSummaryItemTitleWidth" pseudo-property
+        // that represents the current title width of the
+        // expanded/expanding item, or the width of the imaginary
+        // invisible item if we're collapsing everything.
+        Tweener.addTween(this,
+                         { _expandedSummaryItemTitleWidth: this._summaryItemTitleWidth,
+                           time: ANIMATION_TIME,
+                           transition: 'easeOutQuad',
+                           onComplete: this._expandSummaryItemCompleted,
+                           onCompleteScope: this });
+    },
+
+    get _expandedSummaryItemTitleWidth() {
+        if (this._expandedSummaryItem)
+            return this._expandedSummaryItem.getTitleWidth();
         else
-            summaryItem.collapse();
+            return this._imaginarySummaryItemTitleWidth;
+    },
+
+    set _expandedSummaryItemTitleWidth(expansion) {
+        // Expand the expanding item to its new width
+        if (this._expandedSummaryItem)
+            this._expandedSummaryItem.setTitleWidth(expansion);
+        else
+            this._imaginarySummaryItemTitleWidth = expansion;
+
+        // Figure out how much space the other items are currently
+        // using, and how much they need to be shrunk to keep the
+        // total width (including the width of the imaginary item)
+        // constant.
+        let excess = this._summaryItemTitleWidth - expansion;
+        let oldExcess = 0, shrinkage;
+        if (excess) {
+            for (let i = 0; i < this._summaryItems.length; i++) {
+                if (this._summaryItems[i] != this._expandedSummaryItem)
+                    oldExcess += this._summaryItems[i].getTitleWidth();
+            }
+            if (this._expandedSummaryItem)
+                oldExcess += this._imaginarySummaryItemTitleWidth;
+        }
+        if (excess && oldExcess)
+            shrinkage = excess / oldExcess;
+        else
+            shrinkage = 0;
+
+        // Now shrink each one proportionately
+        for (let i = 0; i < this._summaryItems.length; i++) {
+            if (this._summaryItems[i] == this._expandedSummaryItem)
+                continue;
+
+            let width = this._summaryItems[i].getTitleWidth();
+            this._summaryItems[i].setTitleWidth(width * shrinkage);
+        }
+        if (this._expandedSummaryItem)
+            this._imaginarySummaryItemTitleWidth *= shrinkage;
+    },
+
+    _expandSummaryItemCompleted: function() {
+        if (this._expandedSummaryItem)
+            this._expandedSummaryItem.setEllipsization(Pango.EllipsizeMode.END);
     },
 
     _onSummaryItemClicked: function(summaryItem) {
         if (!this._clickedSummaryItem || this._clickedSummaryItem != summaryItem)
             this._clickedSummaryItem = summaryItem;
         else
-            this._clickedSummaryItem = null;
+            this._unsetClickedSummaryItem();
 
         this._updateState();
     },
@@ -1214,7 +1226,7 @@ MessageTray.prototype = {
         let notificationsDone = !notificationsVisible && !notificationsPending;
 
         if (this._summaryState == State.HIDDEN) {
-            if (notificationsDone && this._summaryNeedsToBeShown)
+            if (notificationsDone && this._newSummaryItems.length > 0)
                 this._showSummary(true);
             else if (summarySummoned)
                 this._showSummary(false);
@@ -1438,7 +1450,7 @@ MessageTray.prototype = {
     },
 
     _showSummaryCompleted: function(withTimeout) {
-        this._summaryNeedsToBeShown = false;
+        this._newSummaryItems = [];
 
         if (withTimeout) {
             this._summaryTimeoutId =
@@ -1459,7 +1471,7 @@ MessageTray.prototype = {
                       time: ANIMATION_TIME,
                       transition: 'easeOutQuad'
                     });
-        this._summaryNeedsToBeShown = false;
+        this._newSummaryItems = [];
     },
 
     _showSummaryNotification: function() {
@@ -1469,25 +1481,51 @@ MessageTray.prototype = {
         if (index != -1)
             this._notificationQueue.splice(index, 1);
 
-        this._summaryNotificationBin.child = this._summaryNotification.actor;
+        this._summaryNotificationBoxPointer.bin.child = this._summaryNotification.actor;
         this._summaryNotification.grabFocus(true);
 
-        this._summaryNotificationBin.opacity = 0;
-        this._summaryNotificationBin.y = this.actor.height;
-        this._summaryNotificationBin.show();
-
         if (!this._summaryNotificationExpandedId)
             this._summaryNotificationExpandedId = this._summaryNotification.connect('expanded', Lang.bind(this, this._onSummaryNotificationExpanded));
         this._summaryNotification.expand(false);
+
+        this._clickedSummaryItemAllocationChangedId =
+            this._clickedSummaryItem.actor.connect('allocation-changed',
+                                                   Lang.bind(this, this._adjustNotificationBoxPointerPosition));
+        // _clickedSummaryItem.actor can change absolute postiion without changing allocation
+        this._summaryMotionId = this._summary.connect('allocation-changed',
+                                                      Lang.bind(this, this._adjustNotificationBoxPointerPosition));
+
+        this._summaryNotificationBoxPointer.actor.opacity = 0;
+        this._summaryNotificationBoxPointer.actor.show();
+        this._adjustNotificationBoxPointerPosition();
+
+        this._summaryNotificationState = State.SHOWNING;
+        this._summaryNotificationBoxPointer.animateAppear(Lang.bind(this, function() {
+            this._summaryNotificationState = State.SHOWN;
+        }));
+    },
+
+    _adjustNotificationBoxPointerPosition: function() {
+        // The position of the arrow origin should be the same as center of this._clickedSummaryItem.actor
+        if (!this._clickedSummaryItem)
+            return;
+
+        this._summaryNotificationBoxPointer.setPosition(this._clickedSummaryItem.actor, 0, St.Align.MIDDLE);
+    },
+
+    _unsetClickedSummaryItem: function() {
+        if (this._clickedSummaryItemAllocationChangedId) {
+            this._clickedSummaryItem.actor.disconnect(this._clickedSummaryItemAllocationChangedId);
+            this._summary.disconnect(this._summaryMotionId);
+            this._clickedSummaryItemAllocationChangedId = 0;
+            this._summaryMotionId = 0;
+        }
+
+        this._clickedSummaryItem = null;
     },
 
     _onSummaryNotificationExpanded: function() {
-        this._tween(this._summaryNotificationBin, '_summaryNotificationState', State.SHOWN,
-                    { y: this.actor.height - this._summaryNotificationBin.height,
-                      opacity: 255,
-                      time: ANIMATION_TIME,
-                      transition: 'easeOutQuad'
-                    });
+        this._adjustNotificationBoxPointerPosition();
     },
 
     _hideSummaryNotification: function() {
@@ -1495,25 +1533,18 @@ MessageTray.prototype = {
             this._summaryNotification.disconnect(this._summaryNotificationExpandedId);
             this._summaryNotificationExpandedId = 0;
         }
-
         // Unset this._clickedSummaryItem if we are no longer showing the summary
         if (this._summaryState != State.SHOWN)
-            this._clickedSummaryItem = null;
-        this._summaryNotification.ungrabFocus();
+            this._unsetClickedSummaryItem();
 
-        this._tween(this._summaryNotificationBin, '_summaryNotificationState', State.HIDDEN,
-                    { y: this.actor.height,
-                      opacity: 0,
-                      time: ANIMATION_TIME,
-                      transition: 'easeOutQuad',
-                      onComplete: this._hideSummaryNotificationCompleted,
-                      onCompleteScope: this
-                    });
+        this._summaryNotification.ungrabFocus();
+        this._summaryNotificationState = State.HIDING;
+        this._summaryNotificationBoxPointer.animateDisappear(Lang.bind(this, this._hideSummaryNotificationCompleted));
     },
 
     _hideSummaryNotificationCompleted: function() {
-        this._summaryNotificationBin.hide();
-        this._summaryNotificationBin.child = null;
+        this._summaryNotificationState = State.HIDDEN;
+        this._summaryNotificationBoxPointer.bin.child = null;
         this._summaryNotification.collapseCompleted();
         let summaryNotification = this._summaryNotification;
         this._summaryNotification = null;
