@@ -487,7 +487,7 @@ shell_app_system_get_app (ShellAppSystem   *self,
 /**
  * shell_app_system_get_app_for_path:
  * @system: a #ShellAppSystem
- * @desktop_path: (utf8): UTF-8 encoded absolute file name
+ * @desktop_path: (type utf8): UTF-8 encoded absolute file name
  *
  * Find or create a #ShellApp corresponding to a given absolute
  * file name which must be in the standard paths (XDG_DATA_DIRS).
@@ -629,13 +629,14 @@ shell_app_system_create_from_window (ShellAppSystem *system, MetaWindow *window)
 
 /**
  * shell_app_system_lookup_heuristic_basename:
- * @name: Probable application identifier
+ * @system: a #ShellAppSystem
+ * @id: Probable application identifier
  *
  * Find a valid application corresponding to a given
  * heuristically determined application identifier
  * string, or %NULL if none.
  *
- * Returns: (transfer full): A #ShellApp for name
+ * Returns: (transfer full): A #ShellApp for @name
  */
 ShellApp *
 shell_app_system_lookup_heuristic_basename (ShellAppSystem *system,
@@ -663,9 +664,10 @@ shell_app_system_lookup_heuristic_basename (ShellAppSystem *system,
 
 typedef enum {
   MATCH_NONE,
-  MATCH_MULTIPLE, /* Matches multiple terms */
+  MATCH_SUBSTRING, /* Not prefix, substring */
+  MATCH_MULTIPLE_SUBSTRING, /* Matches multiple criteria with substrings */
   MATCH_PREFIX, /* Strict prefix */
-  MATCH_SUBSTRING /* Not prefix, substring */
+  MATCH_MULTIPLE_PREFIX, /* Matches multiple criteria, at least one prefix */
 } ShellAppInfoSearchMatch;
 
 static char *
@@ -733,36 +735,45 @@ shell_app_info_match_terms (ShellAppInfo  *info,
   match = MATCH_NONE;
   for (iter = terms; iter; iter = iter->next)
     {
+      ShellAppInfoSearchMatch current_match;
       const char *term = iter->data;
       const char *p;
 
+      current_match = MATCH_NONE;
+
       p = strstr (info->casefolded_name, term);
       if (p == info->casefolded_name)
-        {
-          if (match != MATCH_NONE)
-            return MATCH_MULTIPLE;
-          else
-            match = MATCH_PREFIX;
-         }
+        current_match = MATCH_PREFIX;
       else if (p != NULL)
-        match = MATCH_SUBSTRING;
+        current_match = MATCH_SUBSTRING;
 
       p = strstr (info->casefolded_exec, term);
-      if (p == info->casefolded_exec)
-        {
-          if (match != MATCH_NONE)
-            return MATCH_MULTIPLE;
-          else
-            match = MATCH_PREFIX;
-         }
-      else if (p != NULL)
-        match = MATCH_SUBSTRING;
-
-      if (!info->casefolded_description)
-        continue;
-      p = strstr (info->casefolded_description, term);
       if (p != NULL)
-        match = MATCH_SUBSTRING;
+        {
+          if (p == info->casefolded_exec)
+            current_match = (current_match == MATCH_NONE) ? MATCH_PREFIX
+                                                          : MATCH_MULTIPLE_PREFIX;
+          else if (current_match < MATCH_PREFIX)
+            current_match = (current_match == MATCH_NONE) ? MATCH_SUBSTRING
+                                                          : MATCH_MULTIPLE_SUBSTRING;
+        }
+
+      if (info->casefolded_description && current_match < MATCH_PREFIX)
+        {
+          /* Only do substring matches, as prefix matches are not meaningful
+           * enough for descriptions
+           */
+          p = strstr (info->casefolded_description, term);
+          if (p != NULL)
+            current_match = (current_match == MATCH_NONE) ? MATCH_SUBSTRING
+                                                          : MATCH_MULTIPLE_SUBSTRING;
+        }
+
+      if (current_match == MATCH_NONE)
+        return current_match;
+
+      if (current_match > match)
+        match = current_match;
     }
   return match;
 }
@@ -788,14 +799,24 @@ shell_app_info_compare (gconstpointer a,
 
 static GSList *
 sort_and_concat_results (ShellAppSystem *system,
-                         GSList         *multiple_matches,
+                         GSList         *multiple_prefix_matches,
                          GSList         *prefix_matches,
+                         GSList         *multiple_substring_matches,
                          GSList         *substring_matches)
 {
-  multiple_matches = g_slist_sort_with_data (multiple_matches, shell_app_info_compare, system);
-  prefix_matches = g_slist_sort_with_data (prefix_matches, shell_app_info_compare, system);
-  substring_matches = g_slist_sort_with_data (substring_matches, shell_app_info_compare, system);
-  return g_slist_concat (multiple_matches, g_slist_concat (prefix_matches, substring_matches));
+  multiple_prefix_matches = g_slist_sort_with_data (multiple_prefix_matches,
+                                                    shell_app_info_compare,
+                                                    system);
+  prefix_matches = g_slist_sort_with_data (prefix_matches,
+                                           shell_app_info_compare,
+                                           system);
+  multiple_substring_matches = g_slist_sort_with_data (multiple_substring_matches,
+                                                       shell_app_info_compare,
+                                                       system);
+  substring_matches = g_slist_sort_with_data (substring_matches,
+                                              shell_app_info_compare,
+                                              system);
+  return g_slist_concat (multiple_prefix_matches, g_slist_concat (prefix_matches, g_slist_concat (multiple_substring_matches, substring_matches)));
 }
 
 /**
@@ -821,8 +842,9 @@ static inline void
 shell_app_system_do_match (ShellAppSystem   *system,
                            ShellAppInfo     *info,
                            GSList           *terms,
-                           GSList          **multiple_results,
+                           GSList          **multiple_prefix_results,
                            GSList          **prefix_results,
+                           GSList          **multiple_substring_results,
                            GSList          **substring_results)
 {
   const char *id = shell_app_info_get_id (info);
@@ -836,12 +858,17 @@ shell_app_system_do_match (ShellAppSystem   *system,
     {
       case MATCH_NONE:
         break;
-      case MATCH_MULTIPLE:
-        *multiple_results = g_slist_prepend (*multiple_results, (char *) id);
+      case MATCH_MULTIPLE_PREFIX:
+        *multiple_prefix_results = g_slist_prepend (*multiple_prefix_results,
+                                                    (char *) id);
         break;
       case MATCH_PREFIX:
         *prefix_results = g_slist_prepend (*prefix_results, (char *) id);
         break;
+      case MATCH_MULTIPLE_SUBSTRING:
+        *multiple_substring_results = g_slist_prepend (*multiple_substring_results,
+                                                    (char *) id);
+        break;
       case MATCH_SUBSTRING:
         *substring_results = g_slist_prepend (*substring_results, (char *) id);
         break;
@@ -853,8 +880,9 @@ shell_app_system_initial_search_internal (ShellAppSystem  *self,
                                           GSList          *terms,
                                           GSList          *source)
 {
-  GSList *multiple_results = NULL;
+  GSList *multiple_prefix_results = NULL;
   GSList *prefix_results = NULL;
+  GSList *multiple_subtring_results = NULL;
   GSList *substring_results = NULL;
   GSList *iter;
   GSList *normalized_terms = normalize_terms (terms);
@@ -863,19 +891,21 @@ shell_app_system_initial_search_internal (ShellAppSystem  *self,
     {
       ShellAppInfo *info = iter->data;
 
-      shell_app_system_do_match (self, info, normalized_terms, &multiple_results, &prefix_results, &substring_results);
+      shell_app_system_do_match (self, info, normalized_terms,
+                                 &multiple_prefix_results, &prefix_results,
+                                 &multiple_subtring_results, &substring_results);
     }
   g_slist_foreach (normalized_terms, (GFunc)g_free, NULL);
   g_slist_free (normalized_terms);
 
-  return sort_and_concat_results (self, multiple_results, prefix_results, substring_results);
+  return sort_and_concat_results (self, multiple_prefix_results, prefix_results, multiple_subtring_results, substring_results);
 }
 
 /**
  * shell_app_system_initial_search:
- * @self: A #ShellAppSystem
+ * @system: A #ShellAppSystem
  * @prefs: %TRUE if we should search preferences instead of apps
- * @terms: (element-type utf8): List of terms, logical OR
+ * @terms: (element-type utf8): List of terms, logical AND
  *
  * Search through applications for the given search terms.  Note that returned
  * strings are only valid until a return to the main loop.
@@ -893,10 +923,10 @@ shell_app_system_initial_search (ShellAppSystem  *self,
 
 /**
  * shell_app_system_subsearch:
- * @self: A #ShellAppSystem
+ * @system: A #ShellAppSystem
  * @prefs: %TRUE if we should search preferences instead of apps
  * @previous_results: (element-type utf8): List of previous results
- * @terms: (element-type utf8): List of terms, logical OR
+ * @terms: (element-type utf8): List of terms, logical AND
  *
  * Search through a previous result set; for more information, see
  * js/ui/search.js.  Note the value of @prefs must be
@@ -912,8 +942,9 @@ shell_app_system_subsearch (ShellAppSystem   *system,
                             GSList           *terms)
 {
   GSList *iter;
-  GSList *multiple_results = NULL;
+  GSList *multiple_prefix_results = NULL;
   GSList *prefix_results = NULL;
+  GSList *multiple_substring_results = NULL;
   GSList *substring_results = NULL;
   GSList *normalized_terms = normalize_terms (terms);
 
@@ -931,7 +962,9 @@ shell_app_system_subsearch (ShellAppSystem   *system,
       if (!info)
         continue;
 
-      shell_app_system_do_match (system, info, normalized_terms, &multiple_results, &prefix_results, &substring_results);
+      shell_app_system_do_match (system, info, normalized_terms,
+                                 &multiple_prefix_results, &prefix_results,
+                                 &multiple_substring_results, &substring_results);
     }
   g_slist_foreach (normalized_terms, (GFunc)g_free, NULL);
   g_slist_free (normalized_terms);
@@ -939,7 +972,7 @@ shell_app_system_subsearch (ShellAppSystem   *system,
   /* Note that a shorter term might have matched as a prefix, but
      when extended only as a substring, so we have to redo the
      sort rather than reusing the existing ordering */
-  return sort_and_concat_results (system, multiple_results, prefix_results, substring_results);
+  return sort_and_concat_results (system, multiple_prefix_results, prefix_results, multiple_substring_results, substring_results);
 }
 
 const char *
@@ -1250,8 +1283,8 @@ shell_app_info_create_icon_texture (ShellAppInfo *info, float size)
  * shell_app_info_get_source_window:
  * @info: A #ShellAppInfo
  *
- * If @info is tracking a #MetaWindow, return that window.
- * Otherwise, return %NULL.
+ * Returns: (transfer none): If @info is tracking a #MetaWindow,
+ *   return that window.  Otherwise, return %NULL.
  */
 MetaWindow *
 shell_app_info_get_source_window (ShellAppInfo *info)
@@ -1278,7 +1311,6 @@ shell_app_info_launch_full (ShellAppInfo *info,
                             GError      **error)
 {
   GDesktopAppInfo *gapp;
-  char *filename;
   GdkAppLaunchContext *context;
   gboolean ret;
   ShellGlobal *global;
@@ -1300,11 +1332,13 @@ shell_app_info_launch_full (ShellAppInfo *info,
     }
   else if (info->type == SHELL_APP_INFO_TYPE_ENTRY)
     {
-      gapp = g_desktop_app_info_new (shell_app_info_get_id (info));
+      /* Can't use g_desktop_app_info_new, see bug 614879 */
+      const char *filename = gmenu_tree_entry_get_desktop_file_path ((GMenuTreeEntry *)info->entry);
+      gapp = g_desktop_app_info_new_from_filename (filename);
     }
   else
     {
-      filename = shell_app_info_get_desktop_file_path (info);
+      char *filename = shell_app_info_get_desktop_file_path (info);
       gapp = g_desktop_app_info_new_from_filename (filename);
       g_free (filename);
     }
